//////////////////////////////////////////
//renderer.js
//
//This module handles all the drawing calls.
//
//
//Modification History
//when		who		ref		what
//20oct18	ml				written
//
//
//Dependencies:
//Required init.js to be called before this
//and depends on array images[] to exist
//as it uses components of the images[] array
//for animated image strips. In truth this should
//be decoupled but at this stage there is no need to
//
//
//Public Functions and Usage:
//
//
/*
	degtorad(angle)
	//return an angle in radians from an angle specified in degrees
	drawline(sx,sy,fx,fy,color,blend)
	Draws line or plots point with color ('#RRGGBB') and blend mode (blend to be discussed further down)
	drawtext(x,y,text,color,font,centre)
	Draws text at x,y, with color '#RRGGBB' and font eg '20px Arial' at centre true/false
	drawunfilledrect(x,y,w,h,color,blend)
	Draws unfilled rect with colour at x,y and w,h and blend mode
	drawrect(x,y,w,h,color,blend)
	Draws filled rect with colour at x,y, and w,h and blend mode
	drawanimimage(img,x,y,angle,scalex,scaley,framenumber,blend,istext,alpha,index)
	Draws an animated image frame with img at x,y with angle (radians), and scalex/y, using framenumber (0-n), with blend, istext should be false, alpha 0-1, and index = images[] array index number
	drawimage(img,x,y,angle,scalex,scaley,framenumber,blend,istext,alpha,celltype)
	As for drawanimimage except framenumber is ignored, celltype is ignored 
	renderworld()
	Call once per frame at end of all drawing to issue all draw commands to canvas
	flip()
	Call once per frame to flip the canvas into view
	cls(color)
	Clear the screen with colour as shown
	getcolor(r,g,b)
	Return a color in text format '#RRGGBB' from components r,g,b in numeric format
	//
	Blend Mode
	//blend = 0 - standard draw command with no alpha or additive blend mode
	//blend = 1 - draw with additive blend mode
	//blend = 0.05 - 0.99 - draw with alpha between 0 and 1
	//blend = -0.05 - -0.99 - draw with additive blend mode and alpha between 0 and 1
	//
	//screenshake = to make the screen shake (useful for simulating vibration effects
	//on devices without vibration such as a desktop)
	//set this value to approximately 20, and then it is decremented by 1 per frame
	//back to zero - during this time the rendering jitters by the size of the screenshake
	//slightly
*/
//////////////////////////////////////
var overlayalpha = 1;
var renderlist = new Array(); //our array we use to hold or list of items to draw each frame
var screenshake = 0;
var TO_RADIANS = Math.PI/180;
var renderu = new Array(); //a special array used specifically for drawing z-ordered elements
var renderi = 0;
var maxrenderi = 0;
var rendertransition = 0;
var RENDERHALFTRANSIT = 45;
var RENDERTRANSIT = 90;
function degtorad(angle)
{
	return angle * TO_RADIANS;
}

function startrendertransition() //1dec2022
{
	rendertransition = RENDERTRANSIT;
}

function drawline(sx,sy,fx,fy,color,blend)
{
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	if(sx==fx && sy==fy)
	{
		sx--;
		//allows plot of points.
	}
	//sx*=RENDER_GLOBALSX;
	//fx*=RENDER_GLOBALSX;
	//sy*=RENDER_GLOBALSY;
	//fy*=RENDER_GLOBALSY;
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].sx = sx;
	renderlist[renderi].sy = sy;
	renderlist[renderi].fx = fx;
	renderlist[renderi].fy = fy;
	renderlist[renderi].istext = false;
	renderlist[renderi].color = color;
	renderlist[renderi].isline = true;
	renderlist[renderi].isrect = false;
	renderlist[renderi].blend = blend;
	renderlist[renderi].anim = false;	
	renderlist[renderi].sub = false;
	renderi++;
}

function drawtext(x,y,text,color,font,centre)
{
	//put a command to draw text into our render list
	//at a particular x,y with a color and font
	//and a true false flag to indicate if the text should be centred or not
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	var px = font.indexOf('px');
	if(px>0 && isIE==false)
	{
		var size = parseInt(font.substr(0,px));
		//base size = 12.....
		var newsize = (size / 12.0) * RENDER_GLOBALSY;
		font = newsize+'em'+font.substr(px+2,font.length-2);
	}
	x*=RENDER_GLOBALSX;
	y*=RENDER_GLOBALSY;
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].x = x;
	renderlist[renderi].y = y;
	renderlist[renderi].istext = true;
	renderlist[renderi].isline = false;
	renderlist[renderi].color = color;
	renderlist[renderi].font = font;
	renderlist[renderi].text = text;
	renderlist[renderi].centre = centre;
	renderlist[renderi].anim = false;	
	renderlist[renderi].isrect = false;
	renderlist[renderi].sub = false;
	renderi++;
}

function drawunfilledrect(x,y,w,h,color,blend)
{
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	//x*=RENDER_GLOBALSX;
	//y*=RENDER_GLOBALSY;
	//w*=RENDER_GLOBALSX;
	//h*=RENDER_GLOBALSY;
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].isrect = true;
	renderlist[renderi].istext = false;
	renderlist[renderi].x = x;
	renderlist[renderi].y = y;
	renderlist[renderi].w = w;
	renderlist[renderi].h = h;
	renderlist[renderi].isline = false;
	renderlist[renderi].color = color;
	renderlist[renderi].alpha = 1*overlayalpha;
	renderlist[renderi].anim = false;	
	renderlist[renderi].filled = false;	
	renderlist[renderi].sub = false;
	if(blend==0)
	{
		renderlist[renderi].composite = 'source-over';
	}
	else
	{
		if(blend==1)
		{
			renderlist[renderi].composite = 'lighter';
		}
		else
		{
			if(blend<0)
			{
				renderlist[renderi].composite = 'lighter';
				renderlist[renderi].alpha = abs(blend)*overlayalpha;
			}
			else
			{
				renderlist[renderi].composite = 'source-over';
				renderlist[renderi].alpha = blend*overlayalpha;
			}
		}
	}	
	renderi++;
}

function drawrect(x,y,w,h,color,blend)
{
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	//draw a rectangle at x and y with width and height w and h
	//with a color and a blend mode (1 = additive blending)
	//x*=RENDER_GLOBALSX;
	//y*=RENDER_GLOBALSY;
	//w*=RENDER_GLOBALSX;
	//h*=RENDER_GLOBALSY;
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].isrect = true;
	renderlist[renderi].istext = false;
	renderlist[renderi].x = x;
	renderlist[renderi].y = y;
	renderlist[renderi].w = w;
	renderlist[renderi].h = h;
	renderlist[renderi].isline = false;
	renderlist[renderi].color = color;
	renderlist[renderi].alpha = 1*overlayalpha;
	renderlist[renderi].anim = false;	
	renderlist[renderi].filled = true;	
	renderlist[renderi].sub = false;
	if(blend==0)
	{
		renderlist[renderi].composite = 'source-over';
	}
	else
	{
		if(blend==1)
		{
			renderlist[renderi].composite = 'lighter';
		}
		else
		{
			if(blend<0)
			{
				renderlist[renderi].composite = 'lighter';
				renderlist[renderi].alpha = abs(blend)*overlayalpha;
			}
			else
			{
				if(blend==2)
				{
					renderlist[renderi].composite = 'multiply';
					renderlist[renderi].alpha = 1;
				}
				else
				{
					renderlist[renderi].composite = 'source-over';
					renderlist[renderi].alpha = blend*overlayalpha;
				}
				
			}
		}
	}
	renderi++;		
}

function drawsubimage(img,x,y,angle,scalex,scaley,framenumber,blend,istext,alpha,index,ssx,ssy,ssw,ssh)
{
	//relies on images[].img being setup as the index refers to that one.
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	x*=RENDER_GLOBALSX;
	y*=RENDER_GLOBALSY;
	if(!alpha)
	{
		alpha = 1.0;
	}
	istext = false;
	if(!blend)
	{
		blend = 0;
	}
	if(!framenumber)
	{
		framenumber = 0;
	}
	if(!angle)
	{
		angle=0;
	}
	if(!scalex)
	{
		scalex=1;
	}
	if(!scaley)
	{
		scaley=1;
	}
	if(!index)
	{
		index = 0;
	}
	scalex*=RENDER_GLOBALSX;
	scaley*=RENDER_GLOBALSY;
	var celltype = index;
	if(!celltype)
	{
		celltype = 0;
	}
	//NOTE BAD PRACTICE
	//this is coupled with loadimage routine in init.js now
	//decouple it at a later stage
	if(!images[celltype].anim)
	{
		return;
	}
	if(!img)
	{
		return;
	}
	if(img.dataset.anim=='-1')
	{
		drawimage(img,x/RENDER_GLOBALSX,y/RENDER_GLOBALSY,angle,scalex/RENDER_GLOBALSX,scaley/RENDER_GLOBALSY,framenumber,blend,istext,alpha,0);
		return;
	}
	if(overlayalpha<1)
	{
		//scalex*=overlayalpha;
		//scaley*=overlayalpha;
		//angle=(1-overlayalpha)*Math.PI*4;
		//if(ogamestate!=0 && (gamestate<CAT_GAMESTATE_PLAYERTURN || fromstate==CAT_GAMESTATE_DEPLOY)){x = x + (SW - overlayalpha * SW) * overlayalphadirn;}
	}
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].img = img;
	renderlist[renderi].isline = false;
	renderlist[renderi].anim = true;
	renderlist[renderi].isrect = false;
	renderlist[renderi].framenumber = framenumber;
	renderlist[renderi].x = x;
	renderlist[renderi].y = y;
	renderlist[renderi].istext = istext;
	renderlist[renderi].angle = angle;
	renderlist[renderi].scalex = scalex;
	renderlist[renderi].scaley = scaley;
	renderlist[renderi].alpha = alpha*overlayalpha;
	renderlist[renderi].celltype = celltype;
	renderlist[renderi].sub = true;
	renderlist[renderi].subx = ssx;
	renderlist[renderi].suby = ssy;
	renderlist[renderi].subw = ssw;
	renderlist[renderi].subh = ssh;
	
	if(blend==0)
	{
		renderlist[renderi].composite = 'source-over';
	}
	else
	{
		renderlist[renderi].composite = 'lighter';
	}
	renderi++;
}

function drawanimimage(img,x,y,angle,scalex,scaley,framenumber,blend,istext,alpha,index)
{
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	//relies on images[].img being setup as the index refers to that one.
	x*=RENDER_GLOBALSX;
	y*=RENDER_GLOBALSY;
	if(!alpha)
	{
		alpha = 1.0;
	}
	istext = false;
	if(!blend)
	{
		blend = 0;
	}
	if(!framenumber)
	{
		framenumber = 0;
	}
	if(!angle)
	{
		angle=0;
	}
	if(!scalex)
	{
		scalex=1;
	}
	if(!scaley)
	{
		scaley=1;
	}
	if(!index)
	{
		index = 0;
	}
	scalex*=RENDER_GLOBALSX;
	scaley*=RENDER_GLOBALSY;
	var celltype = index;
	if(!celltype)
	{
		celltype = 0;
	}
	//NOTE BAD PRACTICE
	//this is coupled with loadimage routine in init.js now
	//decouple it at a later stage
	if(!images[celltype].anim)
	{
		return;
	}
	if(!img)
	{
		return;
	}
	if(img.dataset.anim=='-1')
	{
		drawimage(img,x/RENDER_GLOBALSX,y/RENDER_GLOBALSY,angle,scalex/RENDER_GLOBALSX,scaley/RENDER_GLOBALSY,framenumber,blend,istext,alpha,0);
		return;
	}
	if(overlayalpha<1)
	{
		//scalex*=overlayalpha;
		//scaley*=overlayalpha;
		//angle=(1-overlayalpha)*Math.PI*4;
		//if(ogamestate!=0 && (gamestate<CAT_GAMESTATE_PLAYERTURN || fromstate==CAT_GAMESTATE_DEPLOY)){x = x + (SW - overlayalpha * SW) * overlayalphadirn;}
	}
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].img = img;
	renderlist[renderi].isline = false;
	renderlist[renderi].anim = true;
	renderlist[renderi].isrect = false;
	renderlist[renderi].framenumber = framenumber;
	renderlist[renderi].x = x;
	renderlist[renderi].y = y;
	renderlist[renderi].sub = false;
	renderlist[renderi].istext = istext;
	renderlist[renderi].angle = angle;
	renderlist[renderi].scalex = scalex;
	renderlist[renderi].scaley = scaley;
	renderlist[renderi].alpha = alpha*overlayalpha;
	renderlist[renderi].celltype = celltype;
	if(blend==0)
	{
		renderlist[renderi].composite = 'source-over';
	}
	else
	{
		if(blend==2)//3apr2022
		{
			renderlist[renderi].composite = 'multiply';
		}
		else
		{
			renderlist[renderi].composite = 'lighter';
		}
	}
	renderi++;
}

function drawimage(img,x,y,angle,scalex,scaley,framenumber,blend,istext,alpha,celltype)
{
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	//framenumber not used currently -> but will be with explosions and stuff however most likely......
	//frame number will need modification for the next game......
	x*=RENDER_GLOBALSX;
	y*=RENDER_GLOBALSY;
	if(!alpha)
	{
		alpha = 1.0;
	}
	istext = false;
	if(!blend)
	{
		blend = 0;
	}
	if(!framenumber)
	{
		framenumber = 0;
	}
	if(!angle)
	{
		angle=0;
	}
	if(!scalex)
	{
		scalex=1;
	}
	if(!scaley)
	{
		scaley=1;
	}
	scalex*=RENDER_GLOBALSX;
	scaley*=RENDER_GLOBALSY;
	if(!celltype)
	{
		celltype = 0;
	}
	if(!img)
	{
		return;
	}
	if(overlayalpha<1)
	{
		//scalex*=overlayalpha;
		//scaley*=overlayalpha;
		//angle=(1-overlayalpha)*Math.PI*4;
		//if(ogamestate!=0 && (gamestate<CAT_GAMESTATE_PLAYERTURN || fromstate==CAT_GAMESTATE_DEPLOY)){x = x + (SW - overlayalpha * SW) * overlayalphadirn;}
	}
	if(img.dataset.anim!='-1')
	{
		drawanimimage(img,x/RENDER_GLOBALSX,y/RENDER_GLOBALSY,angle,scalex/RENDER_GLOBALSX,scaley/RENDER_GLOBALSY,framenumber,blend,istext,alpha,parseInt(img.dataset.anim));
		return;
	}
	if(renderi>=maxrenderi){renderlist[renderi] = new Object;}
	renderlist[renderi].img = img;
	renderlist[renderi].isrect = false;
	renderlist[renderi].framenumber = framenumber;
	renderlist[renderi].x = x;
	renderlist[renderi].y = y;
	renderlist[renderi].istext = istext;
	renderlist[renderi].angle = angle;
	renderlist[renderi].scalex = scalex;
	renderlist[renderi].scaley = scaley;
	renderlist[renderi].isline = false;
	renderlist[renderi].alpha = alpha*overlayalpha;
	renderlist[renderi].celltype = celltype;
	renderlist[renderi].anim = false;	
	renderlist[renderi].sub = false;
	if(blend==0)
	{
		renderlist[renderi].composite = 'source-over';
	}
	else
	{
		renderlist[renderi].composite = 'lighter';
	}
	renderi++;
}
//call renderworld and then flip each frame.....

function renderworld()
{
	//renderworld takes our renderlist and goes through it one by one
	//and draws each image, rect and text to the screen appropriately
	//note - many games use an image atlas to draw sub images from out of a 
	//larger image especially with animation..that can be built in but has not
	//been done with this game
	if(rendertransition>0) //1dec2022
	{
		rendertransition--;
	}
	var sx = 0;
	var sy = 0;
	var sw = 0;
	var sh = 0;
	var sww = 0;
	var shh = 0;
	for(var i=0;i<renderi;i++)
	{
		if(renderlist[i].isrect)
		{
			canvasbuffercontext.save();
			canvasbuffercontext.globalAlpha = renderlist[i].alpha;
			canvasbuffercontext.globalCompositeOperation = renderlist[i].composite;
			if(renderlist[i].filled)
			{
				rect(renderlist[i].x,renderlist[i].y,renderlist[i].w,renderlist[i].h,renderlist[i].color);
			}
			else
			{
				unfilledrect(renderlist[i].x,renderlist[i].y,renderlist[i].w,renderlist[i].h,renderlist[i].color);
			}
			canvasbuffercontext.restore();	
		}
		else
		{
			if(renderlist[i].istext)
			{
				canvasbuffercontext.font = renderlist[i].font;
				canvasbuffercontext.fillStyle = renderlist[i].color;
				if(renderlist[i].centre)
				{
					canvasbuffercontext.textAlign = "center";
				}
				else
				{
					canvasbuffercontext.textAlign = "left";
				}
				canvasbuffercontext.fillText(renderlist[i].text,renderlist[i].x,renderlist[i].y);
			}
			else
			{
				if(renderlist[i].isline)
				{
						canvasbuffercontext.save();
						canvasbuffercontext.globalAlpha = renderlist[i].blend;
						//canvasbuffercontext.globalCompositeOperation = renderlist[i].composite;
						line(renderlist[i].sx,renderlist[i].sy,renderlist[i].fx,renderlist[i].fy,renderlist[i].color);
						canvasbuffercontext.restore();	
				}
				else
				{
					canvasbuffercontext.save();
					canvasbuffercontext.globalAlpha = renderlist[i].alpha;
					canvasbuffercontext.imageSmoothingEnabled=false;
					canvasbuffercontext.translate(renderlist[i].x,renderlist[i].y);
					canvasbuffercontext.rotate(renderlist[i].angle);  //already in radians......using atan2 
					canvasbuffercontext.scale(renderlist[i].scalex,renderlist[i].scaley);
					if(renderlist[i].anim)
					{
						canvasbuffercontext.translate(-images[renderlist[i].celltype].cwx * 0.5,-images[renderlist[i].celltype].chy * 0.5);
					}
					else
					{
						canvasbuffercontext.translate(-renderlist[i].img.width * 0.5,-renderlist[i].img.height * 0.5);
					}
					canvasbuffercontext.globalCompositeOperation = renderlist[i].composite;
					if(renderlist[i].sub)//25mar2022
					{
						canvasbuffercontext.drawImage(renderlist[i].img,renderlist[i].subx,renderlist[i].suby,renderlist[i].subw,renderlist[i].subh,0,0,renderlist[i].subw,renderlist[i].subh);
					}
					else
					{
						if(renderlist[i].anim)
						{
							//NOTE BAD PRACTICE
							//this is coupled with loadimage routine in init.js now
							//decouple it at a later stage
							//set up sx,sy,sw,sh
							//celltype will be set
							sw = images[renderlist[i].celltype].cwx;
							sh = images[renderlist[i].celltype].chy;
							sx = sw * (renderlist[i].framenumber % images[renderlist[i].celltype].cwn);
							sy = sh * (renderlist[i].framenumber - (renderlist[i].framenumber % images[renderlist[i].celltype].cwn))/images[renderlist[i].celltype].cwn;
							canvasbuffercontext.drawImage(renderlist[i].img,sx,sy,sw,sh,0,0,sw,sh);
						}
						else
						{
							canvasbuffercontext.drawImage(renderlist[i].img,0,0);
						}
					}
					canvasbuffercontext.restore();	
				}
			}
		}
	}
	//empty the array afterwards
	//renderlist.splice(0,renderlist.length);
	if(renderi>maxrenderi)
	{
		maxrenderi = renderi;
	}
	if(rendertransition>0) //1dec2022
	{
		if(rendertransition>RENDERHALFTRANSIT)
		{
			//go progressively darker
			canvasbuffercontext.save();
			canvasbuffercontext.globalAlpha = 1;
			canvasbuffercontext.globalCompositeOperation = "multiply";
			var bb = parseInt(255 * ((rendertransition-RENDERHALFTRANSIT)/(RENDERTRANSIT-RENDERHALFTRANSIT)));
			if(bb<10)
			{
				bb = 10;
			}
			if(bb>255)
			{
				bb = 255;
			}
			rect(0,0,SW,SH,getrgb(bb,bb,bb));
			canvasbuffercontext.restore();	
		}
		else
		{
			//go progressively lighter
			canvasbuffercontext.save();
			canvasbuffercontext.globalAlpha = 1;
			canvasbuffercontext.globalCompositeOperation = "multiply";
			var bb = parseInt(255 - 255 * ((rendertransition)/(RENDERHALFTRANSIT)));
			if(bb<10)
			{
				bb = 10;
			}
			if(bb>255)
			{
				bb = 255;
			}
			rect(0,0,SW,SH,getrgb(bb,bb,bb));
			canvasbuffercontext.restore();	
		}
	}
	if(rendertransition>RENDERHALFTRANSIT) //1dec2022
	{
		return;
	}
	renderi = 0;
}

function flip()
{
	//call flip after renderworld to flip the canvasbuffer (backbuffer)
	//into view.
	var offsetx = 0;
	var offsety = 0;
	if(screenshake>0)
	{
		offsetx=RENDER_GLOBALSX*Rand(0,screenshake+1)-screenshake/2;
		offsety=RENDER_GLOBALSY*Rand(0,screenshake+1)-screenshake/2;
		screenshake--;
	}
	canvascontext.drawImage(canvasbuffer,offsetx,offsety);
	/*
	if(RECORDSCREEN)
	{
		var a = document.getElementById('downloadimage');
		var b = a.href;
		a.href = canvasbuffer.toDataURL('image/png');
		a.download = 'images/frame' + (10000000+frame) + '.png';
		a.click();
		a.href = b;
	}
	*/
}

function rect(x,y,w,h,color)
{
	//do not use
	x*=RENDER_GLOBALSX;
	y*=RENDER_GLOBALSY;
	w*=RENDER_GLOBALSX;
	h*=RENDER_GLOBALSY;
	canvasbuffercontext.fillStyle=color;
	canvasbuffercontext.fillRect(x,y,w,h);
}

function unfilledrect(x,y,w,h,color)
{
	x*=RENDER_GLOBALSX;
	y*=RENDER_GLOBALSY;
	w*=RENDER_GLOBALSX;
	h*=RENDER_GLOBALSY;
	canvasbuffercontext.strokeStyle=color;
	canvasbuffercontext.strokeRect(x,y,w,h);
}

function line(sx,sy,fx,fy,color)
{
	sx*=RENDER_GLOBALSX;
	sy*=RENDER_GLOBALSY;
	fx*=RENDER_GLOBALSX;
	fy*=RENDER_GLOBALSY;
	canvasbuffercontext.beginPath();
	canvasbuffercontext.strokeStyle=color;
	canvasbuffercontext.moveTo(sx,sy);
	canvasbuffercontext.lineTo(fx,fy);
	canvasbuffercontext.stroke();
}

function cls(w,h,color)
{
	//do not use
	w*=RENDER_GLOBALSX;
	h*=RENDER_GLOBALSY;
	canvasbuffercontext.fillStyle=color;
	canvasbuffercontext.fillRect(0,0,w,h);
}

function cls(color)
{
	//modify to suit appropriately....
	//use once per frame before drawing
	canvasbuffercontext.fillStyle=color;
	canvasbuffercontext.fillRect(0,0,SW*RENDER_GLOBALSX,SH*RENDER_GLOBALSY);
}

function getrgb(r,g,b)//alias
{
	return getcolor(r,g,b);
}

function getcolor(r,g,b)
{
	//returns a color string of form '#FF0022' for example
	//from components between 0 and 255 for red, green and blue
	var red = "";
	var grn = "";
	var blu = "";
	var clr = "";
	red = String(r.toString(16));
	grn = String(g.toString(16));
	blu = String(b.toString(16));
	if(red.length==0)
	{
		red = "00";
	}
	if(grn.length==0)
	{
		grn = "00";
	}
	if(blu.length==0)
	{
		blu = "00";
	}
	if(red.length==1)
	{
		red = "0" + red;
	}
	if(grn.length==1)
	{
		grn = "0" + grn;
	}
	if(blu.length==1)
	{
		blu = "0" + blu;
	}
	clr = "#" + red + grn + blu;
	return clr;
}
//z-ordered image features

function initu()
{
	//renderu is an array of objects that contains the following features:
	//.cat = index into image
	//.y = y position on screen
	//.x = x position on screen
	//.frame = current animation frame
	renderu = new Array();
}

function clearu()
{
	initu();
}

function resetu(i)
{
	if(renderu.length<i+1)
	{
		renderu[i] = new Object();
	}
	renderu[i].cat = -1;
	renderu[i].y = -1;
	renderu[i].x = -1;
	renderu[i].frame = -1;
}

function setrenderu(cat,x,y,frame)
{
	//called to set parameters needed to draw each sprite
	//called one by one as sprites are desired to be drawn
	//draws in z-order down the screen
	//when drawzorderimages is called
	//after all setrenderu is called
	//for each sprite
	//x*=RENDER_GLOBALSX;
	//y*=RENDER_GLOBALSY;
	resetu(renderu.length);
	renderu[renderu.length-1].cat = cat;
	renderu[renderu.length-1].x = x;
	renderu[renderu.length-1].y = y;
	renderu[renderu.length-1].frame = frame;
}

function drawzorderimages()
{
	//draws images in z-order down the screen
	//with the bottom of the screen being 'closer'
	//to the foreground
	//renderu is an array of objects that contains the following features:
	//.cat = index into image
	//.y = y position on screen
	//.x = x position on screen
	//.frame = current animation frame
	//can draw either animated or non animated units.
	if(renderu.length<=0)
	{
		return;
	}
	var yarray = new Array();
	for(var y=0;y<parseInt(SH*RENDER_GLOBALSY);y++)
	{
		yarray[y] = new Object();
		yarray[y].list = new Array();
	}
	for(var i=0;i<renderu.length;i++)
	{
		if(renderu[i].y>=0 && renderu[i].y<parseInt(SH*RENDER_GLOBALSY))
		{
			var j = yarray[parseInt(renderu[i].y)].list.length;
			yarray[parseInt(renderu[i].y)].list[j] = new Object();
			yarray[parseInt(renderu[i].y)].list[j].cat = renderu[i].cat;
			if(renderu[i].frame>=0)
			{
				yarray[parseInt(renderu[i].y)].list[j].frame = renderu[i].frame;
			}
			else
			{
				yarray[parseInt(renderu[i].y)].list[j].frame = -1;
			}
			yarray[parseInt(renderu[i].y)].list[j].x = renderu[i].x;
			yarray[parseInt(renderu[i].y)].list[j].y = renderu[i].y;
		}
	}
	for(var y=0;y<parseInt(SH*RENDER_GLOBALSY);y++)
	{
		if(yarray[y].list.length>0)
		{
			for(var i=0;i<yarray[y].list.length;i++)
			{
				if(yarray[y].list[i].frame == -1)
				{
					drawimage(images[yarray[y].list[i].cat].img,yarray[y].list[i].x,yarray[y].list[i].y,0,1,1,0,0,0,1,0);
				}
				else
				{
					drawanimimage(images[yarray[y].list[i].cat].img,yarray[y].list[i].x,yarray[y].list[i].y,0,1,1,yarray[y].list[i].frame,0,0,1,yarray[y].list[i].cat);
				}
			}
		}
	}
}

function copyimage(i)
{
	if(i<0 || i>=images.length)
	{
		return -1;
	}
	var count = 0;
	for(var j=0;j<images.length;j++)
	{
		if(images[j].img.src==images[i].img.src)
		{
			count++;
			if(count>1)
			{
				return j;
			}
		}
	}
	var j = -1;
	if(count==1)
	{
		j = images.length;
		images[j] = new Object();
		images[j].cat = 0;
		images[j].img = new Image();
		images[j].img.src = images[i].img.src;
		images[j].img.dataset.anim = j;
		images[j].anim = true;
		if(images[i].anim==false)
		{
			images[j].cwx = parseInt(Math.floor(images[i].img.width / 10));
			images[j].chy = parseInt(Math.floor(images[i].img.height / 10));
			images[j].cwn = 10;
			images[j].chn = 10;
		}
		else
		{
			images[j].cwx = parseInt(Math.floor(images[i].cwx/10));
			images[j].chy = parseInt(Math.floor(images[i].chy/10));
			images[j].cwn = images[i].cwn * 10;
			images[j].chn = images[i].chn * 10;
		}
	}
	else
	{
		return -1;
	}
	return j
}

/*

Assumes these functions exist in init.js

In particular the loadanimimage function

function loadimage(i,file)
{
	images[i] = new Object();
	images[i].cat = 0; //cat is currently not used...ignore this
	images[i].img = new Image();
	images[i].img.src = file

	images[i].img.onload = function(){updateloadscreen()}; //function to call on completing load of each image
	images[i].anim = false;
}

function loadanimimage(i,file,cellwidth,cellheight,cellswide,cellshigh)
{
	images[i] = new Object();
	images[i].cat = 0; //cat is currently not used...ignore this
	images[i].img = new Image();
	images[i].img.src = file

	images[i].img.onload = function(){updateloadscreen()}; //function to call on completing load of each image
	images[i].anim = true;
	images[i].cwx = cellwidth;
	images[i].chy = cellheight;
	images[i].cwn = cellswide;
	images[i].chn = cellshigh;
}
*/
